Attribute VB_Name = "ModUSB"
' Author : Shirish K.
' Copyright Texas Instruments Austin Design Center
' Description: Contains high level functions for USB IO with bq80xx
' Uses bq80xRW2 ActiveX control

Option Explicit
Option Base 0

'Following constants are used only for low level IO
Const MAX_PACKET_SIZE = 256
Const NUM_READ_RETRIES = 3
'Public Bq80xRW1 As Bq80xRW2


'Bq80xRW is the ActiveX object used for bq80xx USB board communication
'It supports High and Low Level IO. Low Level IO functions have not been tested.

Public Function OpenFirstUsedDevice() As Boolean
'Opens the First used(opened) device that the system can find
' For access to opened device in same process space
Dim dummy As String
dummy = "dummy"
If Not Form1.Bq80xRW1.OpenDevice(dummy) Then    'return of 0 = success
    OpenFirstUsedDevice = True              'Success
    Exit Function
End If
MsgBox "Could not open device!Make sure the SMBDialog is started from CCS."
OpenFirstUsedDevice = False     'Failure
End Function
Public Function OpenFirstFreeDevice() As Boolean
'Opens the First FREE(Unopened) device that the system can find
Dim BrdName As String
Dim nBrdsFound As Long
Form1.Bq80xRW1.GetFreeBoards 1, nBrdsFound, BrdName
If nBrdsFound >= 1 Then 'At least 1 board found
    Dim EndPos As Variant
    EndPos = InStr(1, BrdName, ",", vbTextCompare)  'Should always have a "," at the end
    If Not IsNull(EndPos) And EndPos <> 0 Then
        BrdName = Left(BrdName, EndPos - 1)
    End If
    If Not Form1.Bq80xRW1.OpenDevice(BrdName) Then    'return of 0 = success
        OpenFirstFreeDevice = True              'Success
        Exit Function
    End If
End If
OpenFirstFreeDevice = False     'Failure
End Function

Public Sub CloseDevice()
    Form1.Bq80xRW1.CloseDevice
End Sub


Public Function WriteSMBusWord(ByVal ySubCommand As Byte, yMSData As Byte, yLSData As Byte) As Long
    Dim lerror As Long
    lerror = 0
    
    Dim nWord As Integer
    nWord = 0
    If yMSData > 127 Then
        nWord = &H8000
        yMSData = yMSData - 128
    End If
    nWord = nWord + yMSData * 256 + yLSData
    
    lerror = Form1.Bq80xRW1.WriteSMBusWord(ySubCommand, nWord, &H16)  'Non-Zero if error
    If lerror = 0 Then
      lerror = CheckForError
    End If
    WriteSMBusWord = lerror
End Function

Public Function I2CWrite(iByte2 As Integer, iByte3 As Integer, iByte4 As Integer, iByte5 As Integer) As Long
    ' Use default Target address of &HA0
    I2CWrite = Form1.Bq80xRW1.I2CReadWrite(vb_I2C_EE_WR_BYTE, &HA0, iByte3, iByte4)
    ' Parameters for READBYTE are vb_I2C_EE_RD_BYTE, &HA0, WordAddr, VariableToHoldReturnData
    ' Parameters for POWER are vb_I2C_EE_POWER, Data, DummyVariable, DummyVariable
End Function

Public Function WriteSMBusBlock(Command As Integer, DataArray() As Byte, iNumBytes)
    'Dim DataOut(128) As Byte 'This declaration is important for call to WriteSMBusBlock()
    Dim lerror As Integer
    'Dim i As Integer
    'For i = 0 To iNumBytes - 1
        'DataOut(i) = Tcmd.Data(i)
    'Next i
    lerror = Form1.Bq80xRW1.WriteSMBusBlock(Command, DataArray, iNumBytes, &H17)   ' Returns non 0 if error
    If lerror = 0 Then
      lerror = CheckForError
    End If
    WriteSMBusBlock = lerror
End Function
Public Function WriteSMBusCommand(Command As Integer)
Dim lerror As Long
    lerror = Form1.Bq80xRW1.WriteSMBusCmd(Command, &H17)
    If lerror = 0 Then
      lerror = CheckForError
    End If
    WriteSMBusCommand = lerror
End Function

Public Function ReadSMBusWord(ByVal ySubCommand As Byte, yMSData As Byte, yLSData As Byte) As Long
    Dim lerror As Long
    Dim SMBCmd As Integer
    Dim nWord As Integer
    
    SMBCmd = ySubCommand
    lerror = Form1.Bq80xRW1.ReadSMBusWord(SMBCmd, nWord, &H17)
    If lerror = 0 Then  ' Put returned Word into MS and LS byte
        Dim NegSign As Boolean
        NegSign = False
        lerror = 0
        If nWord < 0 Then
            nWord = nWord + 32768
            NegSign = True
        End If
        yMSData = nWord \ 256
        yLSData = nWord - yMSData * 256
        If NegSign Then
            yMSData = yMSData + &H80
        End If
    End If

'    ReportCommErrors lError
    ReadSMBusWord = lerror
End Function

Public Function ReadSMBusBlock(ByVal ySubCommand As Byte, sResponse As String) As Long
    Dim lerror As Long
    Dim Data(128) As Byte
    Dim nLen As Integer
    Dim SubCmd As Integer
    SubCmd = ySubCommand
    'To call ReadSMBusBlock() we need a pointer to variant containing array of bytes
    'So assign an array of bytes to variant v and use v as argument to the call
    Dim v As Variant
    v = Data
    sResponse = ""
    lerror = Form1.Bq80xRW1.ReadSMBusBlock(SubCmd, v, nLen, &H17)
    If lerror = 0 Then  'Stick return data into a string
        Dim X As Integer
        For X = 0 To nLen - 1
            'sResponse = sResponse & Chr(v(X)) This will display actual string for mfg name
            Dim tmpStr As String
            tmpStr = Hex$(v(X))
            If Len(tmpStr) < 2 Then
                tmpStr = "0" & tmpStr
            End If
            sResponse = sResponse & tmpStr
        Next X
    End If
    
'    ReportCommErrors lError
    ReadSMBusBlock = lerror
End Function

Public Function CheckForError() As Integer
   CheckForError = Form1.Bq80xRW1.CheckForError
End Function

